import { createHashHistory as _createHashHistory } from 'history';
import { getQuery, withQuery } from 'ufo';
import { compile, match } from 'path-to-regexp';
import { writable } from 'svelte/store';
export const currentRoute = writable();
const router = _createHashHistory({ window });
let routes = [];
function currentMatched(location) {
    const { pathname, search } = location;
    const query = getQuery(search);
    let params = {};
    const route = routes.find((r) => {
        const fn = match(r.path, { decode: decodeURIComponent });
        const matched = fn(pathname);
        if (matched) {
            params = matched.params;
        }
        return !!matched;
    });
    const notFound = { name: 'NotFound', path: '/404' };
    const _currentRoute = {
        ...(route || notFound),
        path: withQuery(pathname, query),
        query,
        params
    };
    currentRoute.set(_currentRoute);
    return _currentRoute;
}
export function hashChange(listener) {
    return router.listen(({ location, action }) => {
        const _currentRoute = currentMatched(location);
        listener({ currentRoute: _currentRoute, action });
    });
}
function goto(to, replace = false) {
    let path = '';
    if (typeof to === 'string') {
        path = to;
    }
    else {
        let pathname = to.path;
        try {
            const toPath = compile(pathname, { encode: encodeURIComponent });
            pathname = toPath(to.params);
        }
        catch (error) {
            console.error(error);
        }
        path = withQuery(pathname, to.query || {});
    }
    const _path = `${router.location.pathname}${router.location.search}`;
    if (path && path !== _path) {
        if (replace) {
            router.replace(path);
        }
        else {
            router.push(path);
        }
    }
    else {
        console.warn('The current location has not changed.');
    }
}
let unlisten;
export function createHashHistory(setting) {
    routes = setting.routes;
    currentMatched(router.location);
    if (unlisten) {
        unlisten();
    }
    unlisten = router.listen(({ location }) => {
        currentMatched(location);
    });
    const push = (to) => goto(to);
    const replace = (to) => goto(to, true);
    const back = router.back;
    const go = router.go;
    const forward = router.forward;
    return { ...router, push, replace, back, go, forward };
}
